package ww2spring.dao.jdbc;

import java.sql.SQLException;
import java.util.List;

import org.apache.commons.dbutils.QueryRunner;
import org.apache.commons.dbutils.handlers.BeanHandler;
import org.apache.commons.dbutils.handlers.BeanListHandler;

import ww2spring.dao.BookDao;
import ww2spring.entity.Book;

/**
 * DAONX
 */
public class JdbcBookDao extends BaseJdbcDao implements BookDao {

	public Book loadBook(String code) {
		try {
			QueryRunner runner = new QueryRunner(getDataSource());
			Book result = (Book)runner.query(
					"select * from books where code=?", 
					new Object[] {code},
					new BeanHandler(Book.class, JoinedRowProcessor.instance()));
			return result;
		} catch (SQLException e) {
			throw new DataAccessException("JdbcBookDao: unable to execute query", e);
		}
	}

	public List getBooks() {
		try {
			QueryRunner runner = new QueryRunner(getDataSource());
			List result = (List)runner.query(
					"select * from books order by code", 
					new BeanListHandler(Book.class, JoinedRowProcessor.instance()));
			return result;
		} catch (SQLException e) {
			throw new DataAccessException("JdbcBookDao: unable to execute query", e);
		}
	}

	public List getBooksByCategoryCode(String categoryCode) {
		try {
			QueryRunner runner = new QueryRunner(getDataSource());
			List result = (List)runner.query(
					"select * from books where category_code=? order by code", 
					new Object[] {categoryCode},
					new BeanListHandler(Book.class, JoinedRowProcessor.instance()));
			return result;
		} catch (SQLException e) {
			throw new DataAccessException("JdbcBookDao: unable to execute query", e);
		}
	}

	public void updateBook(Book book) {
		try {
			QueryRunner runner = new QueryRunner(getDataSource());
			int updateCount = 
				runner.update(
					"update books " + 
					"set category_code=?, name=?, price=? " +
					"where code=?",
					new String[] { 
						book.getCategoryCode(),
						book.getName(), 
						String.valueOf(book.getPrice()), 
						book.getCode()
					}
				);
			if (updateCount == 0){
				updateCount = 
					runner.update(
						"insert into books (code, category_code, name, price) " +
						"values(?, ?, ?, ?)",
						new String[] { 
							book.getCode(),
							book.getCategoryCode(),
							book.getName(), 
							String.valueOf(book.getPrice())
						}
					);
			}
		} catch (SQLException e) {
			throw new DataAccessException("JdbcBookDao: unable to execute query", e);
		}
		
	}

}