package ww2spring.db;

import java.io.PrintWriter;
import java.sql.Connection;
import java.sql.SQLException;

import javax.sql.DataSource;

import org.apache.commons.dbcp.ConnectionFactory;
import org.apache.commons.dbcp.DriverManagerConnectionFactory;
import org.apache.commons.dbcp.PoolableConnectionFactory;
import org.apache.commons.dbcp.PoolingDataSource;
import org.apache.commons.pool.ObjectPool;
import org.apache.commons.pool.impl.GenericObjectPool;

import ww2spring.service.Configuration;

/**
 * Jakarta Commons-DBCPgf[^\[XB
 */
public class DbcpDataSource implements DataSource {
	
	private DataSource dataSource;
	private Configuration config;
	
	public void setConfiguration(Configuration config) {
		this.config = config;
	}

	private DataSource getDataSouce() {
		if(dataSource == null) {
			dataSource = createDataSource();
		}
		return dataSource;
	}
	
	private DataSource createDataSource() {
		try {
			String driver = config.getProperty("db.driver");
			String url = config.getProperty("db.url");
			String user = config.getProperty("db.user");
			String password = config.getProperty("db.password");
			
			Class.forName(driver);

			ObjectPool connectionPool = new GenericObjectPool(null);
			ConnectionFactory connectionFactory = 
				new DriverManagerConnectionFactory(url, user, password);
			PoolableConnectionFactory poolableConnectionFactory = 
				new PoolableConnectionFactory(connectionFactory, connectionPool, null, null, false, true);
			return new PoolingDataSource(connectionPool);
		} catch (Exception e) {
			e.printStackTrace();
			throw new RuntimeException(e.getMessage(), e);
		}
	}
	
	/* (non-Javadoc)
	 * @see javax.sql.DataSource#getConnection()
	 */
	public Connection getConnection() throws SQLException {
		return getDataSouce().getConnection();
	}

	/* (non-Javadoc)
	 * @see javax.sql.DataSource#getConnection(java.lang.String, java.lang.String)
	 */
	public Connection getConnection(String username, String password)
		throws SQLException {
		return getDataSouce().getConnection(username, password);
	}

	/* (non-Javadoc)
	 * @see javax.sql.DataSource#getLoginTimeout()
	 */
	public int getLoginTimeout() throws SQLException {
		return getDataSouce().getLoginTimeout();
	}

	/* (non-Javadoc)
	 * @see javax.sql.DataSource#getLogWriter()
	 */
	public PrintWriter getLogWriter() throws SQLException {
		return getDataSouce().getLogWriter();
	}

	/* (non-Javadoc)
	 * @see javax.sql.DataSource#setLoginTimeout(int)
	 */
	public void setLoginTimeout(int seconds) throws SQLException {
		getDataSouce().setLoginTimeout(seconds);
	}

	/* (non-Javadoc)
	 * @see javax.sql.DataSource#setLogWriter(java.io.PrintWriter)
	 */
	public void setLogWriter(PrintWriter out) throws SQLException {
		getDataSouce().setLogWriter(out);
	}

}
